/*
   CustomerEJB.java
*/

package J2EEApp;

import java.sql.*;
import javax.sql.*;
import javax.ejb.*;
import javax.naming.*;

public class CustomerEJB implements EntityBean {

    private String customerID;
    private String firstName;
    private String lastName;
    private String address;
    private String phone;

    private EntityContext context;

    public CustomerEJB() {}

    /*
    public String ejbCreate() { return null; }
    public void ejbPostCreate() { }
    */
    public void ejbRemove() throws RemoveException {}

    public void setEntityContext(EntityContext ec) {
        System.out.println("Entering CustomerEJB.setEntityContext()");
        this.context = ec;
        System.out.println("Leaving CustomerEJB.setEntityContext()");
    }

    public void unsetEntityContext() {
        System.out.println("Entering CustomerEJB.unsetEntityContext()");
        System.out.println("Leaving CustomerEJB.unsetEntityContext()");
    }

    public void ejbActivate() {
        System.out.println("Entering CustomerEJB.ejbActivate()");
        customerID = (String)context.getPrimaryKey();
        System.out.println("Leaving CustomerEJB.ejbActivate()");
    }

    public void ejbPassivate() {
        System.out.println("Entering CustomerEJB.ejbPassivate()");
        customerID = null;
        System.out.println("Leaving CustomerEJB.ejbPassivate()");
    }

    public void ejbLoad() {
        System.out.println("Entering CustomerEJB.ejbLoad()");
        String customerID = (String)context.getPrimaryKey();
        try {
            loadRow();
        } catch (Exception ex) {
            throw new EJBException("ejbLoad: " + ex.getMessage());
        }
        System.out.println("Leaving CustomerEJB.ejbLoad()");
    }

    private void loadRow() throws SQLException { // Declared exception caught?
        System.out.println("Entering CustomerEJB.loadRow()");
        // begin: insert new code
	Connection dbConnection = null;
	Statement stmt = null;
	ResultSet rs = null;
	// construct a query to retrieve the customer data from the J2EECUSTOMER table
	String queryStr = "SELECT FIRSTNAME, LASTNAME, ADDRESS, PHONE FROM J2EECUSTOMER " +
	    "WHERE CUSTOMERID = '" + customerID + "'";
	System.out.println("queryString is: "+ queryStr);
	try {
	    // get a database connection
	    dbConnection = initDBConnection();
	    // create a statement using dbConnection
	    stmt = dbConnection.createStatement();
	    // execute the query
	    rs = stmt.executeQuery(queryStr);
	    if (rs != null && rs.next()) {
		// initialize the EJB's persistent fields with the ResuleSet values
		firstName = rs.getString("FIRSTNAME");
		lastName  = rs.getString("LASTNAME");
		address   = rs.getString("ADDRESS");
		phone     = rs.getString("PHONE");
	    }
	} catch (SQLException sqlex) {
	    // throw a NoSuchEntityException if no row is found for the customer
	    throw new NoSuchEntityException(sqlex.getMessage());
	} finally {
	    rs.close();
	    // close the statement object
	    stmt.close();
	    // close the database connection
	    dbConnection.close();
	}
        // end: insert new code
        System.out.println("Leaving CustomerEJB.loadRow()");
    }

    public void ejbStore() {
        System.out.println("Entering CustomerEJB.ejbStore()");
        // begin: insert new code
        // retrieve the primary key from the EntityContext
        String customerID = (String)context.getPrimaryKey();
	try {
	    storeRow();
	} catch (Exception ex) {
        // handle any exceptions by throwing an EJBException
            throw new EJBException("ejbStore: " + ex.getMessage());
	}
        // end: insert new code
        System.out.println("Leaving CustomerEJB.ejbStore()");
    }

    private void storeRow() throws SQLException {
        System.out.println("Entering CustomerEJB.storeRow()");
        Connection dbConnection = initDBConnection();
        Statement stmt = dbConnection.createStatement();
        String queryStr = "UPDATE J2EECUSTOMER SET"
            + " firstname = '" +  firstName.trim()
            + "', lastname = '" + lastName.trim()
            + "', address = '" + address.trim()
            + "', phone = '" + phone.trim()
            + "' WHERE customerid = " + "'" + customerID + "'";
        System.out.println("queryString is: "+ queryStr);
        int resultCount = stmt.executeUpdate(queryStr);
        stmt.close();
        dbConnection.close();
        if (resultCount == 0) {
            throw new EJBException("Storing row for customerID " +
                                    customerID + " failed.");
        }
        System.out.println("Leaving CustomerEJB.storeRow()");
    }

    public String ejbFindByPrimaryKey (String key) throws FinderException {
        System.out.println("Entering CustomerEJB.ejbFindByPrimaryKey()");
        customerID = key;
        try {
        if (!selectByPrimaryKey(customerID))
            throw new ObjectNotFoundException("Row for id " + key +
                                                " not found.");
        } catch (Exception ex) {
            throw new EJBException("ejbFindByPrimaryKey: " + ex.getMessage());
        }
        System.out.println("Leaving CustomerEJB.ejbFindByPrimaryKey()");
        return customerID;
    }

    private boolean selectByPrimaryKey(String key) throws SQLException {
        System.out.println("Entering CustomerEJB.selectByPrimaryKey(" + key + ")");
        // begin: insert new code
	ResultSet rs = null;
	boolean result = false;
        Connection dbConnection = null;
        Statement stmt = null;
        // construct a query to check for the existence of the customer in the J2EECUSTOMER table
	String queryStr = "SELECT FIRSTNAME, LASTNAME, ADDRESS, PHONE FROM J2EECUSTOMER " +
	    "WHERE CUSTOMERID = '" + key + "'";
	System.out.println("queryString is: "+ queryStr);
	try {
	    // get a database connection
	    dbConnection = initDBConnection();
	    // create a statement using dbConnection
	    stmt = dbConnection.createStatement();
	    // execute the query
	    rs = stmt.executeQuery(queryStr);
	    if (rs != null && rs.next()) {
		// initialize the EJB's persistent fields with the ResuleSet values
		firstName = rs.getString("FIRSTNAME");
		lastName  = rs.getString("LASTNAME");
		address   = rs.getString("ADDRESS");
		phone     = rs.getString("PHONE");
		result = true;
	    }
	} catch (SQLException sqlex) {
	    // throw a NoSuchEntityException if no row is found for the customer
	    throw new NoSuchEntityException(sqlex.getMessage());
	} finally {
	    rs.close();
	    // close the statement object
	    stmt.close();
	    // close the database connection
	    dbConnection.close();
	    return result;
	}
	/*
	String queryStr = "SELECT CUSTOMERID FROM J2EECUSTOMER WHERE CUSTOMERID = '" + key + "'";
        System.out.println("queryString is: "+ queryStr);
	try {
	// execute the query
	    rs = stmt.executeQuery(queryStr);
	} catch (Exception e) {
	    System.out.println("Exception: " + e.getMessage());
	} finally {
        // close the statement object
	    stmt.close();
        // close the database connection
	    dbConnection.close();
        // return true if a row is found; false otherwise
	    if (rs != null && rs.next()) result = true;

	    // end: insert new code
	    System.out.println("Leaving CustomerEJB.selectByPrimaryKey() result=" + result);
	    return result;
	}
	*/
    }

    // business methods
    public CustomerData getCustomerData() {
        return (new CustomerData(customerID,firstName,lastName,phone));
    }

    // utility method
    private Connection initDBConnection() throws SQLException {
        System.out.println("Entering CustomerEJB.initDBConnection()");
        Connection c = null;
        try {
            InitialContext ic = new InitialContext();
            DataSource ds =
                (DataSource)ic.lookup("java:comp/env/jdbc/BankMgrDB");
            c = ds.getConnection();
        } catch (NamingException ne) { throw new EJBException(ne); }
        catch (SQLException se) { throw new EJBException(se); }
        System.out.println("Leaving CustomerEJB.initDBConnection()");
        return c;
    }
}



