/*
 * AccountEJB.java
 */

package J2EEApp;

import java.sql.*;
import java.util.*;

import javax.naming.*;
import javax.ejb.*;
import javax.sql.*;

public class AccountEJB implements EntityBean {

    private String acctNumber;
    private String customerID;
    private String description;
    private double balance;

    private Vector accounts;

    private EntityContext context;

    public AccountEJB() { System.out.println("***AccountEJB constructor...");  }

    public String ejbCreate (String acctNumber, String customerID,
                             String description, double balance)
        throws DuplicateKeyException,CreateException {
        System.out.println("--==>Entering AccountEJB.ejbCreate()");
        // begin: insert new code
        // initilize the bean's persistent fields using the method arguments
        this.acctNumber = acctNumber;
        this.customerID = customerID;
        this.description = description;
        this.balance = balance;
        // end: insert new code
        AccountDAO ADAO = new AccountDAO();
        // begin: insert new code
        // initilize the DAO's fields using the method arguments
        ADAO.setCustomerID(this.customerID);
        ADAO.setDescription(this.description);
        ADAO.setAcctNumber(this.acctNumber);
        ADAO.setBalance(this.balance);
        // end: insert new code
        try{
            Connection dbConnection = getDBConnection();
            ADAO.create(dbConnection);
            dbConnection.close();
        } catch (java.sql.SQLException se) {
            throw new CreateException ("--==>SQL Exception in create:" + se);
        }
        System.out.println("--==>Leaving AccountEJB.ejbCreate()");
        return (acctNumber);
    }

    public String ejbCreate (String customerID,
                             String description, 
                             double balance)
        throws DuplicateKeyException,CreateException 
    {
        System.out.println("--==>Entering AccountEJB.ejbCreate() with Sequencer lookup");

        this.acctNumber = getNextAcctSequenceValue(); // Exercise 8
        this.customerID = customerID;
        this.description = description;
        this.balance = balance;

        AccountDAO ADAO = new AccountDAO();

        ADAO.setCustomerID(this.customerID);
        ADAO.setDescription(this.description);
        ADAO.setAcctNumber(this.acctNumber);
        ADAO.setBalance(this.balance);

        try{
            Connection dbConnection = getDBConnection();
            ADAO.create(dbConnection);
            dbConnection.close();
        } catch (java.sql.SQLException se) {
            throw new CreateException ("SQL Exception in create:" + se);
        }
        System.out.println("--==>Leaving AccountEJB.ejbCreate() with Sequencer lookup");
        return (this.acctNumber);
    }

    public void setEntityContext(EntityContext ec) {
        System.out.println("--==>Entering AccountEJB.setEntityContext()");
        this.context = ec;
        System.out.println("--==>Leaving AccountEJB.setEntityContext()");
    }

    public void unsetEntityContext() {
        System.out.println("--==>Entering AccountEJB.unsetEntityContext()");
        System.out.println("--==>Leaving AccountEJB.unsetEntityContext()");
    }

    public void ejbRemove() throws RemoveException {
        System.out.println("--==>Entering AccountEJB.ejbRemove()");
        AccountDAO ADAO = new AccountDAO();
        // begin: insert new code
        // initilize the DAO's acctNumber field with the EJB's primary key value
        ADAO.setAcctNumber((String)context.getPrimaryKey());
        // end: insert new code
        try{
            // begin: insert new code
            // get a database connection
            Connection dbConnection = getDBConnection();
            // call the DAO's remove method
            ADAO.remove(dbConnection);
            // close the database connection
            dbConnection.close();
            // end: insert new code
        } catch (java.sql.SQLException se) {
            throw new RemoveException ("SQL Exception in remove");
        }
        System.out.println("--==>Leaving AccountEJB.ejbRemove()");
    }

    public void ejbLoad() {
        System.out.println("--==>Entering AccountEJB.ejbLoad()");
        // begin: insert new code
        // create a new DAO named ADAO
        AccountDAO ADAO = new AccountDAO();
        // initilize the DAO's acctNumber field with the EJB's primary key value
        ADAO.setAcctNumber((String)context.getPrimaryKey());
        // end: insert new code
        try{
            Connection dbConnection = getDBConnection();
            ADAO.load(dbConnection);
            dbConnection.close();
        } catch (java.sql.SQLException se) {
            throw new EJBException (se);
        }
        // begin: insert new code
        // set the EJB's persistent fields to the values of the corresponding DAO fields
        this.acctNumber = ADAO.getAcctNumber();
        this.customerID = ADAO.getCustomerID();
        this.description = ADAO.getDescription();
        this.balance = ADAO.getBalance();
        // end: insert new code
        System.out.println("--==>Leaving AccountEJB.ejbLoad()");
    }

    public void ejbStore() {
        System.out.println("--==>Entering AccountEJB.ejbStore()");
        AccountDAO ADAO = new AccountDAO();
        // begin: insert new code
        // initialize the DAO's acctNumber field with the bean's primary key value
        ADAO.setAcctNumber((String)context.getPrimaryKey());
        // initilize the other DAO fields using the EJB's persistent field values
        ADAO.setCustomerID(this.customerID);
        ADAO.setDescription(this.description);
        ADAO.setBalance(this.balance);
        // end: insert new code
        try{
            Connection dbConnection = getDBConnection();
            ADAO.store(dbConnection);
            dbConnection.close();
        } catch (java.sql.SQLException se) {
            throw new EJBException (se);
        }
        System.out.println("--==>Leaving AccountEJB.ejbStore()");
    }

    public void ejbActivate() {
        System.out.println("--==>Entering AccountEJB.ejbActivate()");
        acctNumber = (String)context.getPrimaryKey();
        System.out.println("--==>Leaving AccountEJB.ejbActivate()");
    }

    public void ejbPassivate() {
        System.out.println("--==>Entering AccountEJB.ejbPassivate()");
        acctNumber = null;
        System.out.println("--==>Leaving AccountEJB.ejbPassivate()");
    }

    /* Exercise 6, Section 4 */
    public Collection ejbFindByCustomerID (String customerID)
        throws FinderException
    {
        System.out.println("--==>Entering AccountEJB.ejbFindByCustomerID()");
        Collection collection = null;
        AccountDAO ADAO = new AccountDAO();
        ADAO.setCustomerID(customerID);
        try{
            Connection dbConnection = getDBConnection();
            collection = ADAO.findByCustomerID(dbConnection);
            dbConnection.close();
        } catch (java.sql.SQLException se) {
            throw new FinderException ("AccountEJB.ejbFindByCustomerID: " + se.getMessage());
        } finally {
            System.out.println("--==>tLeaving AccountEJB.ejbFindByCustomerID()");
            return collection;
        }
    }
    public String ejbFindByPrimaryKey(String key)
        throws FinderException {
        System.out.println("--==>Entering AccountEJB.ejbFindByPrimaryKey()");
        AccountDAO ADAO = new AccountDAO();
        ADAO.setAcctNumber(key);
        String theKey = null;
        try{
            Connection dbConnection = getDBConnection();
            theKey = ADAO.findByPrimaryKey(dbConnection);
            dbConnection.close();
        } catch (java.sql.SQLException se) {
            throw new FinderException ("SQL Exception in find by primary key");
        }
        System.out.println("--==>Leaving AccountEJB.ejbFindByPrimaryKey()");
        return theKey;
    }

    public void ejbPostCreate(String acctNumber, String customerID,
                              String description, double balance)
        throws DuplicateKeyException,CreateException {
    }

    public void ejbPostCreate(String customerID, String description,
                              double balance)
        throws DuplicateKeyException,CreateException {
    }

    // business methods
    public AccountData getAccountData() {
        return (new AccountData(acctNumber, balance, description));
    }

    // utility methods
    private String getNextAcctSequenceValue()
    {
        System.out.println("--==>--->Entering AccountEJB.getNextAcctSequenceValue()");
        String nextInSequence = "100"; // dummy default
        try {
            System.out.println("--==>--->In try{}");
            InitialContext ic = new InitialContext();
            System.out.println("--==>--->new InitialContext created, calling lookup()");
            SequencerHome mySequencerHome = (SequencerHome)ic.lookup("sequencerServer");
            System.out.println("--==>--->lookup of sequencerServer, calling findByPrimaryKey()");
            Sequencer sequencer = mySequencerHome.findByPrimaryKey("ACCOUNT");
            System.out.println("--==>--->retrieved Sequencer object by PrimaryKey");
            nextInSequence = sequencer.getNextSequenceValue();
            System.out.println("--==>--->post sequencer.getNextSequenceValue()");
        } catch (Exception e) {
            System.out.println("--==>--->Exception in AccountEJB.getNextAcctSequenceValue: " +
                               e.getMessage());
            e.printStackTrace();
            System.out.println("--==>--->default value of nextInSequence unchanged from " + nextInSequence);
        } finally {
            System.out.println("--==>--->Leaving AccountEJB.getNextAcctSequenceValue: " + nextInSequence);
            System.out.flush();
            return nextInSequence;
        }
    }

    private Connection getDBConnection() throws SQLException {
        System.out.println("--==>Entering AccountEJB.getDBConnection()");
        Connection connection;
        try {
            InitialContext ic = new InitialContext();
            DataSource ds = (DataSource)ic.lookup("java:comp/env/jdbc/BankMgrDB");
            connection = ds.getConnection();
        } catch (NamingException ne) { throw new EJBException(ne); }
        catch (SQLException se) { throw new EJBException(se); }
        System.out.println("--==>Leaving AccountEJB.getDBConnection()");
        return connection;
    }

    private void dumpStrings(Collection c)
    {
        System.out.println("--==>Entering AccountEJB.dumpStrings()");
        Iterator iterator = c.iterator();
        int count = 0;
        while (iterator.hasNext()) {
            String s = (String)iterator.next();
            System.out.println("--==>\tElement " + count + ": " + s);
            count++;
        }
        System.out.println("--==>Leaving AccountEJB.dumpStrings()");
    }
}
