package J2EEApp;

import java.sql.*;
import java.util.*;

import javax.sql.*;
import javax.naming.*;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.rmi.PortableRemoteObject;

public class BankMgrEJB implements SessionBean
{
    private static final String loginQuery = "SELECT USERID FROM J2EEAPPUSER WHERE USERNAME = ? AND PASSWORD = ?";
    private javax.sql.DataSource jdbcFactory;
    private CustomerHome myCustomerHome;

    private Customer customer = null;
    private CustomerData custData = null;
    private CustomerWithAcctsData cwad = null;
    private Vector customers = null;

    // SessionBean implementation
    public void ejbCreate() 
    {
	System.out.println("Entering BankMgrEJB.ejbCreate()");
	try {
	    Context initialContext = new InitialContext();
	    jdbcFactory = (DataSource)initialContext.lookup("java:comp/env/jdbc/BankMgrDB");
	    Object result = initialContext.lookup("customerServer");
	    myCustomerHome = (CustomerHome)PortableRemoteObject.narrow(result,CustomerHome.class);
	} catch (NamingException ne) {
	    System.out.println("NamingException:\n" + ne.getMessage());
	} finally {
	    System.out.println("Leaving BankMgrEJB.ejbCreate()");
	}
    }
    public void setSessionContext(SessionContext ctx) { }

    public void ejbActivate() { }

    public void ejbPassivate() { }

    public void ejbRemove() { }


    public String loginUser(String pUsername, String pPassword)
    {
	System.out.println("BankMgrEJB.loginUser: " + pUsername + ", " + pPassword + "\n");

	String userID = null;
	Connection conn = null;
	PreparedStatement ps = null;
	ResultSet rs = null;
	try {
	    conn = jdbcFactory.getConnection();
	    ps = conn.prepareStatement(loginQuery);
	    ps.setString(1, pUsername);
	    ps.setString(2, pPassword);
	    rs = ps.executeQuery();
	    if (rs !=  null && rs.next()) {
		userID = rs.getString("USERID").trim();
		
	    } else {
		System.out.println("loginUser: No USERID found\n");
	    }
	} catch (SQLException se) {
	    System.out.println("SQL Exception: " + se.getMessage());
	    se.printStackTrace();
	} catch (Exception e) {
	    System.out.println("Exception: " + e.getMessage());
	    e.printStackTrace();
	} finally {
	    try {
		if (rs != null) rs.close();
		if (ps != null) ps.close();
		if (conn != null) conn.close();
	    } catch (SQLException se) { 
		System.out.println("BankMgrEJB.loginUser: SQL Exception after close - Ignored");
	    }
	}
	return userID;
    } // loginUser

    // Bean methods
    public Vector getAllCustomers()
    {
	System.out.println("Entering BankMgrEJB.getAllCustomers()");
	customers = new Vector();
	try {
	    Collection cl = myCustomerHome.findAll();
	    if (cl != null) {
		Iterator cli = cl.iterator();
		while (cli.hasNext()) {
		    customers.addElement(((Customer)cli.next()).getCustomerData()); 
		}
		System.out.println("--=>Envoking dumpCustomers");
		dumpCustomers(customers);
	    } else {
		System.out.println("--=>CustomerHome.findAll returns null collection");
	    }
	} catch (Exception e) {
	    System.out.println("Exception: " + e.getMessage());
	    e.printStackTrace();  
	} finally {
	    System.out.println("Leaving BankMgrEJB.getAllCustomers()");
	    return customers;
	}
    }

    public CustomerData getCustomerData(String customerID)
    {
	System.out.println("Entering BankMgrEJB.getCustomerData()");
	customer = null;
	custData = null;
	try {
	    customer = myCustomerHome.findByPrimaryKey(customerID);
	    System.out.println("Got Customer from findByPrimaryKey()");
	    custData = customer.getCustomerData();
	} catch (Exception e) {
	    System.out.println("Bad ID " + customerID + ": " + e);
	} finally {
	    System.out.println("Leaving BankMgrEJB.getCustomerData()");
	    return custData;
	}
    } // getCustomerData

    public CustomerWithAcctsData getCustomerWithAcctsData(String customerID)
    {
	System.out.println("---=> Entering BankMgrEJB.getCustomersWithAcctsData() for " + 
			   customerID);
	customer = null;
	cwad = null;
	try {
	    customer = myCustomerHome.findByPrimaryKey(customerID);
	    if (customer != null) {
		System.out.println("---=> Created Customer object");
		cwad = customer.getCustomerWithAcctsData();
		System.out.println("---=> Created CustomerWithAcctsData  object");
	    }
	} catch (Exception e) {
	    System.out.println("BankMgrEJB.getCustomerWithAcctsData Exception: " + e.getMessage());
	    e.printStackTrace();
	} finally {
	    System.out.println("---=> Leaving BankMgrEJB.getCustomersWithAcctsData() for " + 
			       customerID);
	    return cwad;
	}
    }

    private void dumpStrings(Collection c)
    {
	System.out.println("Entering BankMgrEJB.dumpStrings()");
	Iterator iterator = c.iterator();
	int count = 0;
	while (iterator.hasNext()) {
	    String s = (String)iterator.next();
	    System.out.println("\tElement " + count + ": " + s);
	    count++;
	}
	System.out.println("Leaving BankMgrEJB.dumpStrings()");
    }

    private void dumpCustomers(Vector c)
    {
	System.out.println("Entering BankMgrEJB.dumpCustomers()");
	Iterator iterator = c.iterator();
	int count = 0;
	while (iterator.hasNext()) {
	    CustomerData s = (CustomerData)iterator.next();
	    System.out.println("\tElement " + count + ": " + s);
	    count++;
	}
	System.out.println("Leaving BankMgrEJB.dumpCustomers()");
    }

} // end class
