/*
   CustomerEJB.java
*/

package J2EEApp;

import java.sql.*;
import java.util.*;

import javax.sql.*;
import javax.ejb.*;
import javax.naming.*;
import javax.rmi.*;

public class CustomerEJB implements EntityBean {

    private String customerID;
    private String firstName;
    private String lastName;
    private String address;
    private String phone;

    private Vector acctsData;
    private Vector customers;

    private EntityContext context;

    public CustomerEJB() 
    { 
	System.out.println("***Creating a CustomerEJB object...");
    }


    public String ejbCreate(String key) 
	throws CreateException 
    { 
	customerID = key; 
	return null;
    }
    public void ejbPostCreate(String key) { }


    public void ejbRemove() throws RemoveException {}

    public void setEntityContext(EntityContext ec) {
        System.out.println("-=>Entering CustomerEJB.setEntityContext()");
        this.context = ec;
        System.out.println("-=>Leaving CustomerEJB.setEntityContext()");
    }

    public void unsetEntityContext() {
        System.out.println("-=>Entering CustomerEJB.unsetEntityContext()");
	context = null;
        System.out.println("-=>Leaving CustomerEJB.unsetEntityContext()");
    }

    public void ejbActivate() {
        System.out.println("-=>Entering CustomerEJB.ejbActivate()");
        customerID = (String)context.getPrimaryKey();
        System.out.println("-=>Leaving CustomerEJB.ejbActivate()");
    }

    public void ejbPassivate() {
        System.out.println("-=>Entering CustomerEJB.ejbPassivate()");
        customerID = null;
        System.out.println("-=>Leaving CustomerEJB.ejbPassivate()");
    }

    public void ejbLoad() {
        System.out.println("-=>Entering CustomerEJB.ejbLoad()");
        String customerID = (String)context.getPrimaryKey();
        try {
            loadRow();
        } catch (Exception ex) {
            throw new EJBException("ejbLoad: " + ex.getMessage());
        }
        System.out.println("-=>Leaving CustomerEJB.ejbLoad()");
    }

    private void loadRow() throws SQLException { // Declared exception caught?
        System.out.println("-=>Entering CustomerEJB.loadRow()");
        // begin: insert new code
        Connection dbConnection = null;
        Statement stmt = null;
        ResultSet rs = null;
        // construct a query to retrieve the customer data from the J2EECUSTOMER table
        String queryStr = "SELECT FIRSTNAME, LASTNAME, ADDRESS, PHONE " +
	    " FROM J2EECUSTOMER " +
            "WHERE CUSTOMERID = '" + customerID + "'";
        System.out.println("-=>queryString is: "+ queryStr);
        try {
            // get a database connection
            dbConnection = initDBConnection();
            // create a statement using dbConnection
            stmt = dbConnection.createStatement();
            // execute the query
            rs = stmt.executeQuery(queryStr);
            if (rs != null && rs.next()) {
                // initialize the EJB's persistent fields with the ResuleSet values
                firstName = (rs.getString("FIRSTNAME")).trim();
                lastName  = (rs.getString("LASTNAME")).trim();
                address   = (rs.getString("ADDRESS")).trim();
                phone     = (rs.getString("PHONE")).trim();
            }
        } catch (SQLException sqlex) {
            // throw a NoSuchEntityException if no row is found for the customer
            throw new NoSuchEntityException(sqlex.getMessage());
        } finally {
            // close the statement object
            stmt.close();
            // close the database connection
            dbConnection.close();
        }
        // end: insert new code
        System.out.println("-=>Leaving CustomerEJB.loadRow()");
    }

    public void ejbStore() {
        System.out.println("-=>Entering CustomerEJB.ejbStore()");
        // begin: insert new code
        // retrieve the primary key from the EntityContext
        String customerID = (String)context.getPrimaryKey();
        try {
            storeRow();
        } catch (Exception ex) {
        // handle any exceptions by throwing an EJBException
            throw new EJBException("CustomerEJB.ejbStore: " + ex.getMessage());
        }
        // end: insert new code
        System.out.println("-=>Leaving CustomerEJB.ejbStore()");
    }

    private void storeRow() throws SQLException {
        System.out.println("-=>Entering CustomerEJB.storeRow()");
        Connection dbConnection = initDBConnection();
        Statement stmt = dbConnection.createStatement();
        String queryStr = "UPDATE J2EECUSTOMER SET"
            + " firstname = '" +  firstName.trim()
            + "', lastname = '" + lastName.trim()
            + "', address = '" + address.trim()
            + "', phone = '" + phone.trim()
            + "' WHERE customerid = " + "'" + customerID + "'";
        System.out.println("-=>queryString is: "+ queryStr);
        int resultCount = stmt.executeUpdate(queryStr);
        stmt.close();
        dbConnection.close();
        if (resultCount == 0) {
            throw new EJBException("Storing row for customerID " +
                                    customerID + " failed.");
        }
        System.out.println("-=>Leaving CustomerEJB.storeRow()");
    }

    public String ejbFindByPrimaryKey (String key) throws FinderException {
        System.out.println("-=>Entering CustomerEJB.ejbFindByPrimaryKey()");
        customerID = key;
        try {
        if (!selectByPrimaryKey(customerID))
            throw new ObjectNotFoundException("Row for id " + key +
                                                " not found.");
        } catch (Exception ex) {
            throw new EJBException("ejbFindByPrimaryKey: " + ex.getMessage());
        } 
        System.out.println("-=>Leaving CustomerEJB.ejbFindByPrimaryKey()");
        return customerID;
    }

    private boolean selectByPrimaryKey(String key) throws SQLException {
        System.out.println("-=>Entering CustomerEJB.selectByPrimaryKey(" + key + ")");
        // begin: insert new code
        ResultSet rs = null;
        boolean result = false;
        Connection dbConnection = null;
        Statement stmt = null;
        // construct a query to check for the existence of the customer in the J2EECUSTOMER table
        String queryStr = "SELECT CUSTOMERID FROM J2EECUSTOMER " +
            "WHERE CUSTOMERID = '" + key + "'";
        System.out.println("-=>queryString is: "+ queryStr);
        try {
            // get a database connection
            dbConnection = initDBConnection();
            // create a statement using dbConnection
            stmt = dbConnection.createStatement();
            // execute the query
            rs = stmt.executeQuery(queryStr);
	    result = rs.next();	// true if we found something
	    /*
            if (rs != null && rs.next()) {
                // initialize the EJB's persistent fields with the ResuleSet values
                firstName = rs.getString("FIRSTNAME");
                lastName  = rs.getString("LASTNAME");
                address   = rs.getString("ADDRESS");
                phone     = rs.getString("PHONE");
            }
	    */
	} catch (SQLException sqlex) {
	    System.out.println("CustomerEJB.selectByPrimaryKey SQL Exception:" 
			       + sqlex.getMessage());
	    sqlex.printStackTrace();
        } finally {
            // close the statement object
            stmt.close();
            // close the database connection
            dbConnection.close();
            return result;
        }
    }

    /*
     * Added Exercise 6
     */
    public Collection ejbFindAll() throws FinderException
    {
        System.out.println("-=>Entering CustomerEJB.ejbFindAll()");

	try {
	    Collection col = selectAll();
	    if (col.isEmpty())
		System.out.println("-=>No customers found in ejbFindALL()");
	    return col;
	} catch (Exception ex) {
	    throw new EJBException("CustomerEJB.ejbFindAll: " + ex.getMessage());
	}
    }

    private Collection selectAll()
	throws SQLException
    {
        Connection dbConnection = initDBConnection();
	System.out.println("-=>Creating Statement");
        Statement stmt = dbConnection.createStatement();
        String queryStr = "SELECT * FROM J2EECUSTOMER ORDER BY CUSTOMERID";
        System.out.println("-=>queryString is: "+ queryStr);
	System.out.println("-=>Executing Query");
        ResultSet rs = stmt.executeQuery(queryStr);
	ArrayList v =  new ArrayList();
	while (rs.next()) {
	    String customerID = rs.getString("CUSTOMERID");
	    System.out.println("-=>Adding CustomerID " + customerID);
	    v.add(customerID);
	}
	stmt.close();
	dbConnection.close();
	return v;
    }

    public CustomerWithAcctsData getCustomerWithAcctsData()
    {
        System.out.println("-=>Entering CustomerEJB.getCustomerWithAcctsData() customerID=" + customerID);
        System.out.flush();
        acctsData = new Vector();
        try {
            System.out.println("-=>CustomerEJB.getCustomerWithAcctsData: creating InitialContext");
            InitialContext ic = new InitialContext();
            System.out.println("-=>CustomerEJB.getCustomerWithAcctsData: creating AccountHome");

	    Object result = ic.lookup("accountServer");
            AccountHome myAccountHome = 
		(AccountHome)PortableRemoteObject.narrow(result, AccountHome.class);

            Collection acctNumbers = myAccountHome.findByCustomerID(this.customerID);
	    if (acctNumbers == null) {
		acctsData = null;
	    } else {
		System.out.println("-=>CustomerEJB.getCustomerWithAcctsData: preparing Iterator");
		Iterator itr = acctNumbers.iterator();
		while (itr.hasNext()) {
                    acctsData.addElement(((Account)itr.next()).getAccountData());
		    /*
		    System.out.println("-=>CustomerEJB.getCustomerWithAcctsData: Iterator hasNext");
		    String acctNum = (String)itr.next();
		    System.out.println("-=>Retreiving acctNum = " + acctNum);
		    Account acct  = myAccountHome.findByPrimaryKey(acctNum);
		    System.out.println("-=>Created AccountEJB, calling getAccountData");
		    AccountData data = acct.getAccountData();
		    System.out.println("-=>Return from getAccountData()");
		    System.out.print("-=>CustomerEJB.getCustomerWithAcctsData:\nAdding ");
		    System.out.println(data.toString());
		    acctsData.addElement(data);
		    */
		}
	    }
        } catch (Exception e) {
            System.out.println("-=>\n?Exception in CustomerEJB.getCustomerWithAcctsData: " +
                               e.getMessage());
            e.printStackTrace();
            System.out.flush();
        } finally {
            System.out.println("-=>Leaving CustomerEJB.getCustomerWithAcctsData()");
            System.out.flush();
            return new CustomerWithAcctsData(this.getCustomerData(),acctsData);
        }
    }


    // business methods
    public CustomerData getCustomerData() {
        return (new CustomerData(customerID,firstName,lastName,address, phone));
    }

    // utility method
    private Connection initDBConnection() throws SQLException {
        System.out.println("-=>Entering CustomerEJB.initDBConnection()");
        Connection c = null;
        try {
            InitialContext ic = new InitialContext();
            DataSource ds =
                (DataSource)ic.lookup("java:comp/env/jdbc/BankMgrDB");
            c = ds.getConnection();
        } catch (NamingException ne) { throw new EJBException(ne); }
        catch (SQLException se) { throw new EJBException(se); }
        System.out.println("-=>Leaving CustomerEJB.initDBConnection()");
        return c;
    }

  private void dumpCustomers(Vector c)
    {
        System.out.println("-=>Entering CustomerEJB.dumpCustomers()");
        Iterator iterator = c.iterator();
        int count = 0;
        while (iterator.hasNext()) {
            CustomerData s = (CustomerData)iterator.next();
            System.out.println("-=>Element " + count + ": " + s);
            count++;
        }
        System.out.println("-=>Leaving CustomerEJB.dumpCustomers()");
    }

    private void dumpStrings(Collection c)
    {
        System.out.flush();
        System.out.println("-=>Entering CustomerEJB.dumpStrings()");
        Iterator iterator = c.iterator();
        int count = 0;
        while (iterator.hasNext()) {
            String s = (String)iterator.next();
            System.out.println("-=>\tElement " + count + ": " + s);
            count++;
        }
        System.out.println("-=>Leaving CustomerEJB.dumpStrings()");
        System.out.flush();
    }

}
